#!/usr/bin/perl
#
######################################################################
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#######################################################################
#
# this script is the frontend called from bin/$terminal/$script
# all the accounting modules are linked to this script which in
# turn execute the same script in bin/$terminal/
#
#######################################################################

# setup defaults, these are overidden by sql-ledger.conf
# DO NOT CHANGE
$userspath = "users";
$templates = "templates";
$memberfile = "users/members";
$sendmail = "| /usr/sbin/sendmail -t";
########## end ###########################################


$| = 1;

use SL::Form;

eval { require "sql-ledger.conf"; };


$form = new Form;


# name of this script
$pos = rindex $0, '/';
$script = substr($0, $pos + 1);

# we use $script for the language module
$form->{script} = $script;
# strip .pl for translation files
$script =~ s/\.pl//;

# pull in DBI
use DBI;

# check for user config file, could be missing or ???
eval { require("$userspath/$form->{login}.conf"); };
$form->error($@) if ($@);


$myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};

if ($form->{action} ne "save_preferences") {
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);
}


# locale messages
$locale = new Locale "$myconfig{countrycode}", "$script";


# check password
$form->error($locale->text('Incorrect Password!')) if ($form->{password} ne $myconfig{password});
 

$form->{path} =~ s/\.\.\///g;
if ($form->{path} !~ /^bin\//) {
  $form->error($locale->text('Invalid path!')."\n");
}

# did sysadmin lock us out
if (-e "$userspath/nologin") {
  $form->error($locale->text('System currently down for maintenance!'));
}


# pull in the main code
require "$form->{path}/$form->{script}";

# customized scripts
if (-f "$form->{path}/custom_$form->{script}") {
  eval { require "$form->{path}/custom_$form->{script}"; };
  $form->error($@) if ($@);
}

# customized scripts for login
if (-f "$form->{path}/$form->{login}_$form->{script}") {
  eval { require "$form->{path}/$form->{login}_$form->{script}"; };
  $form->error($@) if ($@);
}


if ($form->{action}) {
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error($locale->text('action= not defined!'));
}


# end
